package com.ho.analysing.log;

import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * <p>Title: LogAnalyzer</p>
 * 
 * Display the error present in a log file.
 */
public class LogAnalyzer {

	/**
	 * Contains the number of each error
	 */
	private static Map<String, Long> errorCount = new HashMap<String, Long>();

	/**
	 * Increment the number of occurrences of a given error. 
	 */
	private static void incrementErrorCount(String label) {
		Long currCount = errorCount.get(label);
		if (null == currCount) {
			currCount = 0L;
		} // end-if
		currCount++;
		errorCount.put(label, currCount);
	}
	
	/**
	 * Display on the console occurrences of errors.
	 */
	private static void displayReport(File logFile, ArrayList<String> errorsKeyword, ArrayList<String> errorsToSkip, Calendar startCal, Calendar endCal) {
		System.out.println("\n\n---------- [ Report ] for "+logFile.getName()+" (size:"+logFile.length()+")------------- ");
		System.out.println("Analysing started at "+startCal.getTime().toString()+" and ended at "+endCal.getTime().toString());
		long millis = Math.abs(endCal.getTime().getTime() - startCal.getTime().getTime());
		String executionTimeStr = String.format("%d min, %d sec, %d ms",      
				TimeUnit.MILLISECONDS.toMinutes(millis),     
				TimeUnit.MILLISECONDS.toSeconds(millis) - TimeUnit.MINUTES.toSeconds(TimeUnit.MILLISECONDS.toMinutes(millis)),
				millis 	- TimeUnit.MILLISECONDS.toMillis(TimeUnit.MILLISECONDS.toMinutes(millis))
						- TimeUnit.MILLISECONDS.toMillis(TimeUnit.MILLISECONDS.toSeconds(millis) - TimeUnit.MINUTES.toSeconds(TimeUnit.MILLISECONDS.toMinutes(millis)))
				); 

		System.out.println("Execution time is ("+millis+"ms): " + executionTimeStr);   
		
		System.out.println("Error Keywords:");
		for (Iterator<String> iterator = errorsKeyword.iterator(); iterator.hasNext();) {
			String errorKeyword = iterator.next();
			System.out.println("- "+errorKeyword);
		}
		System.out.println("\nError to skip:");
		for (Iterator<String> iterator = errorsToSkip.iterator(); iterator.hasNext();) {
			String errorToSkip = iterator.next();
			System.out.println("- "+errorToSkip);
		}
		
		System.out.println("\nError count : ");
		for (String currKey : errorCount.keySet()) {
			System.out.println(errorCount.get(currKey) + " occurences of : " + currKey);
		} // end-for
	}

	
	public static void analyzeLog(String logFileNameAndPath, ArrayList<String> errorsKeyword, ArrayList<String> errorsToSkip) {
		if(logFileNameAndPath==null || errorsKeyword==null || errorsToSkip==null){
			throw new RuntimeException("Some parameters are null!!!!!!");
		}

		File logFile = new File(logFileNameAndPath);

		// Errors keywords
		{
			for (Iterator<String> iterator = errorsKeyword.iterator(); iterator.hasNext();) {
				String errorKeyword = iterator.next();
				errorCount.put(errorKeyword,new Long(0L));
			}
			
		} // end-block

		Calendar startCal = GregorianCalendar.getInstance();
		
		try {
			FileInputStream fstream = new FileInputStream(logFile);
			DataInputStream in = new DataInputStream(fstream);
	        BufferedReader br = new BufferedReader(new InputStreamReader(in));
	        
	        int MAX_NUMBER_OF_PREVIOUS_LINES = 5;
	        ArrayList<String> previousLines =  new ArrayList<String>();
	        
			String strLine;
		    while ((strLine = br.readLine()) != null)   {

		    	// Determines if the current line contains a keyword error
		    	boolean processCurrentLine = false;
		    	{
		    		for (String currErrStr : errorsKeyword) {
		    			if (-1 != strLine.indexOf(currErrStr)) {
		    				processCurrentLine = true;
		    				LogAnalyzer.incrementErrorCount(currErrStr);
		    			} // end-if
		    		} // end-for
		    	} // end-block
		    	
		    	// IF the current line contains the keyword ERROR
		    	// THEN displays the previous MAX_NUMBER_OF_PREVIOUS_LINES lines and displays the current line
		    	if (processCurrentLine) {

		    		// Checks if the error should be ignored or not
		    		boolean skipError = false;
		    		for (String currErrorToSkip : errorsToSkip) {
		    			if (-1 != strLine.indexOf(currErrorToSkip)) {
		    				skipError = true;
		    				LogAnalyzer.incrementErrorCount(currErrorToSkip);
		    				break;
		    			} // end-block
		    		} // end-for
		    		
		    		if (!skipError) {
			    		System.out.println("\n--------------------- [error found with "+MAX_NUMBER_OF_PREVIOUS_LINES+" previous lines] ----------------------");
			    		for (String currPreviousLine : previousLines) {
			    			System.out.println(currPreviousLine);
			    		} // end-for
	
			    		// Displays the previous MAX_NUMBER_OF_PREVIOUS_LINES lines 
			    		{
			    			System.out.println (">>>>>>>>" + strLine);
			    			String currTmpLine = null;
			    			for (int i=0; i < MAX_NUMBER_OF_PREVIOUS_LINES && null != (currTmpLine = br.readLine()) ;i++ ) {
			    				System.out.println(currTmpLine);
			    			} // end-block
			    		} // end-block
		    		} // end-if
		    		
		    	} // end-if
		    	
		    	// Holds the last MAX_NUMBER_OF_PREVIOUS_LINES lines of log
		    	{
			    	if (previousLines.size()>MAX_NUMBER_OF_PREVIOUS_LINES) {
			    		previousLines.remove(0);
			    	} // end-if
			    	previousLines.add(strLine);
		    	} // end-while
		    	
		    } // end-while
		    in.close();
		} catch (Exception e) {
			e.printStackTrace();
		} // end-try
		
		Calendar endCal = GregorianCalendar.getInstance();
		//
		displayReport(logFile, errorsKeyword, errorsToSkip, startCal, endCal);
	}

	public static void main(String[] args) {
		
		String baseDir = Thread.currentThread().getContextClassLoader().getResource("jUnitResources").getPath();
		{
			String logFileNameAndPath = baseDir+File.separator+"server.log";
			// Errors keywords
			ArrayList<String> errorsKeyword = new ArrayList<String>();
			errorsKeyword.add("ERROR [");
			// Errors to skip
			ArrayList<String> errorsToSkip = new ArrayList<String>();
			errorsToSkip.add("ERROR [STDERR]");
			analyzeLog(logFileNameAndPath, errorsKeyword, errorsToSkip);
		}
/*		{
			String logFileNameAndPath = baseDir+File.separator+"easyphp.log";
			// Errors keywords
			ArrayList<String> errorsKeyword = new ArrayList<String>();
			errorsKeyword.add(" error ");
			// Errors to skip
			ArrayList<String> errorsToSkip = new ArrayList<String>();
			errorsToSkip.add("error 5");
			analyzeLog(logFileNameAndPath, errorsKeyword, errorsToSkip);
		}
		{
			String logFileNameAndPath = baseDir+File.separator+"error.log";
			// Errors keywords
			ArrayList<String> errorsKeyword = new ArrayList<String>();
			errorsKeyword.add("[error] ");
			errorsKeyword.add("Fatal error:");
			// Errors to skip
			ArrayList<String> errorsToSkip = new ArrayList<String>();
			errorsToSkip.add("PHP Fatal error:  Call to undefined method PDF");
			analyzeLog(logFileNameAndPath, errorsKeyword, errorsToSkip);
		}

		{
			String logFileNameAndPath = baseDir+File.separator+"access.log";
			// Errors keywords
			ArrayList<String> errorsKeyword = new ArrayList<String>();
			errorsKeyword.add("GET /home/mysql/css/phpmyadmin.css.php");
			// Errors to skip
			ArrayList<String> errorsToSkip = new ArrayList<String>();
			analyzeLog(logFileNameAndPath, errorsKeyword, errorsToSkip);
		}
		{
			String logFileNameAndPath = baseDir+File.separator+"tomcathuo.log";
			// Errors keywords
			ArrayList<String> errorsKeyword = new ArrayList<String>();
			errorsKeyword.add("ERROR [");
			errorsKeyword.add("ORA-");
			// Errors to skip
			ArrayList<String> errorsToSkip = new ArrayList<String>();
			errorsToSkip.add("11:00:32,886 ERROR [");
			analyzeLog(logFileNameAndPath, errorsKeyword, errorsToSkip);
		}
*/
	}
	

	
}
